<?php

namespace Drupal\fixed_text_link_formatter\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\link\Plugin\Field\FieldFormatter\LinkFormatter;

/**
 * Plugin implementation of the 'link' formatter.
 *
 * @FieldFormatter(
 *   id = "fixed_text_link",
 *   label = @Translation("Link with fixed text"),
 *   field_types = {
 *     "link"
 *   }
 * )
 */
class FixedTextLink extends LinkFormatter {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return array(
      'trim_length' => '',
      'link_text' => t('View website'),
    ) + parent::defaultSettings();
  }

  public function settingsForm(array $parentForm, FormStateInterface $form_state) {
    $parentForm = parent::settingsForm($parentForm, $form_state);
    $settings = $this->getSettings();

    unset($parentForm['trim_length']);
    unset($parentForm['url_only']);
    unset($parentForm['url_plain']);

    $form['link_text'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Link text'),
      '#default_value' => $settings['link_text'],
      '#required' => TRUE,
    ];

    return $form + $parentForm;
  }

  public function settingsSummary() {
    $settings = $this->getSettings();

    $summary[] = $this->t('Link text: @text', ['@text' => $settings['link_text']]);
    if (!empty($settings['rel'])) {
      $summary[] = $this->t('Add rel="@rel"', ['@rel' => $settings['rel']]);
    }
    if (!empty($settings['target'])) {
      $summary[] = $this->t('Open link in new window');
    }

    return $summary;
  }

  public function viewElements(FieldItemListInterface $items, $langcode) {
    $settings = $this->getSettings();

    foreach ($items as $item) {
      $item->set('title', $settings['link_text']);
    }

    return parent::viewElements($items, $langcode);
  }
}
